﻿# ImportVcards2Outlook
# 20200629-HH
#
# Zweck:
# Import von mehr als einem Eintrag in einer VCard-Datei in das persoenliche
# Adressbuch in Outlook
# (Anlass: Migration von Thunderbird o.ae. nach Outlook)
#
# Funktionsweise:
# Schleife ueber einzelne VCards einer VCards-Datei mit ggf. mehreren VCards,
# in der Schleife erzeugen einer temp. Einzel-VCard-Datei und 
# Import dieser in Outook
#
# Voraussetzung:
# Outlook muss auf dem Rechner installiert sein


BEGIN 
	{
		# Herstellen der Verbindung zu Outlook
		try {
			Add-Type -assembly "Microsoft.Office.Interop.Outlook" -ErrorAction Stop -ErrorVariable "OutlookError"
			$Outlook=New-Object -comobject Outlook.Application -ErrorAction stop -ErrorVariable "ApplicationError"
			$mapiNS=$Outlook.GetNameSpace("MAPI")
			#$contactFolder=$mapiNS.GetDefaultFolder([Microsoft.Office.Interop.Outlook.OlDefaultFolders]::olFolderContacts)
		}
		catch {
			$OutlookError
			$ApplicationError
		}
		finally {
		}

		# Oeffnen der als Parameter uebergebenen vcf-Datei
		$vcFile = $args[0]
		if ($vcFile -eq $NULL) {
			Write-Host Usage: .\ImportVcards2Outlook.ps1 filename.vcf
			Write-Host Examp: .\ImportVcards2Outlook.ps1 c:\exportet.vcf
			Exit
		}

		# Unterordner funktioniert nicht wie gehofft:
		#$importContactFolder=$contactFolder.Folders.Add("vcfImport")
		#$importContactFolder.ShowAsOutlookAB=$TRUE
		# (aber wg. Problemen mit move (s.u.) sowieso egal...
		#$importContactFolder=$contactFolder
		$TmpFile=[System.IO.Path]::GetTempFileName()  # endet auf .tmp, jetzt kommt dirty-hack:
		$vcTmpFile=$TmpFile.Replace('.tmp','.vcf')
		Rename-Item $TmpFile $vcTmpFile
		$count=0
		$validEntry=$FALSE # ein echter Eintrag ist noch nicht begonnen worden
		$outEncoding="OEM" # ASCII schneidet Sonderzeichen weg
		[Console]::InputEncoding = [Text.UTF8Encoding]::UTF8
	}

PROCESS
	{
		Write-Host Einlesen der VCard-Datei $vcFile
		switch -regex -file $vcFile {
			"^BEGIN:VCARD" {
				$count++
				$validEntry=$TRUE # richtig eingeleitet, bis zum naechsten END
				Set-Content -Encoding $outEncoding -path $vcTmpFile -value $_
			}
			"^END:VCARD" {
				if ($validEntry) {
					Add-Content -Encoding $outEncoding -path $vcTmpFile -value $_
					$validEntry=$FALSE # bis zum naechsten BEGIN
					# Einlesen der temp. vcf mit einem Eintrag
					$singleContact=$mapiNS.OpenSharedItem($vcTmpFile)
						# vgl. https://docs.microsoft.com/de-de/office/vba/api/Outlook.NameSpace.OpenSharedItem
					# Move funktioniert zwar, liefert aber Fehlermeldung ...
					#[void]$singleContact.move($importContactFolder)
						# vgl. https://docs.microsoft.com/de-de/office/vba/api/outlook.contactitem.move
					# aber einfach in Default-Ordner fuer Kontakte funktioniert:
					$singleContact.save()
						# vgl. https://docs.microsoft.com/en-us/office/vba/api/outlook.contactitem.save
				}
			}
			"^FN:" { # fast wie default, aber ...
				if ($validEntry) {
					Write-Output $_.Substring(3) # ... FN zur Info ausgeben
					Add-Content -Encoding $outEncoding -path $vcTmpFile -value $_
				}
			}
			default { # ist der Inhalt einer VCard ...
				if ($validEntry) { # ... sonst head, foot, comment etc. ignorieren
					Add-Content -Encoding $outEncoding -path $vcTmpFile -value $_
				}
			}
		}
	}

END 
	{
	 	Remove-Item $vcTmpFile
		Write-Host Es wurden $count Eintraege importiert
	}

# EOF
